Package: wikitaxa
Title: Taxonomic Information from 'Wikipedia'
Description: 'Taxonomic' information from 'Wikipedia', 'Wikicommons',
    'Wikispecies', and 'Wikidata'. Functions included for getting
    taxonomic information from each of the sources just listed, as
    well performing taxonomic search.
Version: 0.5.0
License: MIT + file LICENSE
URL: https://docs.ropensci.org/wikitaxa/,
        https://github.com/ropensci/wikitaxa
BugReports: https://github.com/ropensci/wikitaxa/issues
Authors@R: c(
    person("Scott", "Chamberlain", role = "aut",
      email = "myrmecocystus+r@gmail.com"),
    person("Ethan", "Welty", role = "aut"),
    person("Grzegorz", "Sapijaszko", role = "aut"),
    person("Zachary", "Foster", role = c("aut", "cre"),
      email = "zacharyfoster1989@gmail.com")
  )
LazyLoad: yes
LazyData: yes
Encoding: UTF-8
Language: en-US
VignetteBuilder: knitr
Depends: R(>= 3.2.1)
Imports: data.table, curl, crul (>= 0.3.4), tibble, jsonlite, xml2,
        WikipediR, httr
Suggests: testthat, knitr, rmarkdown, vcr
RoxygenNote: 7.3.3
X-schema.org-applicationCategory: Taxonomy
X-schema.org-keywords: taxonomy, species, API, web-services, Wikipedia,
        vernacular, Wikispecies, Wikicommons
X-schema.org-isPartOf: https://ropensci.org
NeedsCompilation: no
Packaged: 2026-02-12 17:49:41 UTC; fosterz
Author: Scott Chamberlain [aut],
  Ethan Welty [aut],
  Grzegorz Sapijaszko [aut],
  Zachary Foster [aut, cre]
Maintainer: Zachary Foster <zacharyfoster1989@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-13 09:40:03 UTC

# Generated by roxygen2: do not edit by hand

S3method(wt_data,default)
S3method(wt_data,wiki_id)
export(wt_data)
export(wt_data_id)
export(wt_wiki_page)
export(wt_wiki_page_parse)
export(wt_wiki_url_build)
export(wt_wiki_url_parse)
export(wt_wikicommons)
export(wt_wikicommons_parse)
export(wt_wikicommons_search)
export(wt_wikipedia)
export(wt_wikipedia_parse)
export(wt_wikipedia_search)
export(wt_wikispecies)
export(wt_wikispecies_parse)
export(wt_wikispecies_search)

# wikitaxa

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![cran
checks](https://badges.cranchecks.info/worst/wikitaxa.svg)](https://CRAN.r-project.org/package=wikitaxa)
[![R-CMD-check](https://github.com/ropensci/wikitaxa/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ropensci/wikitaxa/actions/workflows/R-CMD-check.yaml)
[![codecov](https://codecov.io/gh/ropensci/wikitaxa/branch/master/graph/badge.svg)](https://app.codecov.io/gh/ropensci/wikitaxa)
[![rstudio mirror
downloads](https://cranlogs.r-pkg.org/badges/wikitaxa)](https://github.com/r-hub/cranlogs.app)
[![cran
version](https://www.r-pkg.org/badges/version/wikitaxa)](https://cran.r-project.org/package=wikitaxa)

`wikitaxa` - taxonomy data from Wikipedia/Wikidata/Wikispecies

`wikitaxa` docs: <https://docs.ropensci.org/wikitaxa/>

See also the taxize book: <https://books.ropensci.org/taxize/>

### Low level API

The low level API is meant for power users and gives you more control,
but requires more knowledge.

- `wt_wiki_page()`
- `wt_wiki_page_parse()`
- `wt_wiki_url_build()`
- `wt_wiki_url_parse()`
- `wt_wikispecies_parse()`
- `wt_wikicommons_parse()`
- `wt_wikipedia_parse()`

### High level API

The high level API is meant to be easier and faster to use.

- `wt_data()`
- `wt_data_id()`
- `wt_wikispecies()`
- `wt_wikicommons()`
- `wt_wikipedia()`

Search functions:

- `wt_wikicommons_search()`
- `wt_wikispecies_search()`
- `wt_wikipedia_search()`

## Installation

CRAN version

``` r
install.packages("wikitaxa")
```

Dev version

``` r
remotes::install_github("ropensci/wikitaxa")
```

``` r
library('wikitaxa')
```

## Contributors

- [Ethan Welty](https://github.com/ezwelty)
- [Scott Chamberlain](https://github.com/sckott)
- [Zachary S.L. Foster](https://github.com/zachary-foster)

## Meta

- Please [report any issues or
  bugs](https://github.com/ropensci/wikitaxa/issues).
- License: MIT
- Get citation information for `wikitaxa` in R doing
  `citation(package = 'wikitaxa')`
- Please note that this project is released with a [Contributor Code of
  Conduct](https://github.com/ropensci/wikitaxa/blob/master/CODE_OF_CONDUCT.md).
  By participating in this project you agree to abide by its terms.

[![ropensci](https://ropensci.org/public_images/github_footer.png)](https://ropensci.org)

#' Wikidata taxonomy data
#'
#' @export
#' @param x (character) a taxonomic name
#' @param property (character) a property id, e.g., P486
#' @param ... curl options passed on to `httr::GET()`
#' @param language (character) two letter language code
#' @param limit (integer) records to return. Default: 10
#' @return `wt_data` searches Wikidata, and returns a list with elements:
#' \itemize{
#'  \item labels - data.frame with columns: language, value
#'  \item descriptions - data.frame with columns: language, value
#'  \item aliases - data.frame with columns: language, value
#'  \item sitelinks - data.frame with columns: site, title
#'  \item claims - data.frame with columns: claims, property_value,
#'  property_description, value (comma separated values in string)
#' }
#'
#' `wt_data_id` gets the Wikidata ID for the searched term, and
#' returns the ID as character
#'
#' @details Note that `wt_data` can take a while to run since when fetching
#' claims it has to do so one at a time for each claim
#'
#' You can search things other than taxonomic names with `wt_data` if you
#' like
#' @examples \dontrun{
#' # search by taxon name
#' wt_data("Mimulus alsinoides")
#'
#' # choose which properties to return
#' wt_data(x="Mimulus foliatus", property = c("P846", "P815"))
#'
#' # get a taxonomic identifier
#' wt_data_id("Mimulus foliatus")
#'
#' # the id can be passed directly to wt_data()
#' wt_data(wt_data_id("Mimulus foliatus"))
#' }
wt_data <- function(x, property = NULL, ...) {
  UseMethod("wt_data")
}

#' @export
wt_data.wiki_id <- function(x, property = NULL, ...) {
  data_wiki(x, property = property, ...)
}

#' @export
wt_data.default <- function(x, property = NULL, ...) {
  result <- WikipediR::query(
    url = "https://www.wikidata.org/w/api.php",
    out_class = "list",
    clean_response = FALSE,
    query_param = list(
      action   = "wbsearchentities",
      type     = "item",
      language = "en",
      limit    = 10,
      search   = x
    ),
    ...
  )
  if (length(result$search) == 0) stop("no results found", call. = FALSE)
  data_wiki(result$search[[1]]$id, property = property, ...)
}

#' @export
#' @rdname wt_data
wt_data_id <- function(x, language = "en", limit = 10, ...) {
  result <- WikipediR::query(
    url = "https://www.wikidata.org/w/api.php",
    out_class = "list",
    clean_response = FALSE,
    query_param = list(
      action   = "wbsearchentities",
      type     = "item",
      language = language,
      limit    = limit,
      search   = x
    ),
    ...
  )
  out <- result$search
  out <- if (length(out) == 0) NA else out[[1]]$id
  structure(out, class = "wiki_id")
}

data_wiki <- function(x, property = NULL, ...) {
  x <- sub(x,pattern = '^([0-9]+)', replacement = 'Q\\1')
  xx <- lapply(x, function(page, ...){
    result <- WikipediR::page_content(
      domain = "wikidata.org",
      page_name = page,
      as_wikitext = TRUE,
      httr::user_agent("wikitaxa"),
      ...
    )
    out <- jsonlite::fromJSON(result$parse$wikitext[[1]])
    return(out)
  })

  if (is.null(property)) {
    claims <- create_claims(xx[[1]]$claims)
  } else{
    cl <- Filter(function(x) any(x$mainsnak$property %in% property), xx[[1]]$claims)
    if (length(cl) == 0) stop("No matching properties", call. = FALSE)
    claims <- create_claims(cl)
  }

  list(
    labels = dt_df(xx[[1]]$labels),
    descriptions = dt_df(xx[[1]]$descriptions),
    aliases = dt_df(xx[[1]]$aliases),
    sitelinks = dt_df(lapply(xx[[1]]$sitelinks, function(x)
      x[names(x) %in% c('site', 'title')])),
    claims = dt_df(claims)
  )
}

fetch_property <- function(x) {
  x <- sub(x,pattern = '^([0-9]+)', replacement = 'P\\1')
  x <- sub(x,pattern = '^(P[0-9]+)', replacement = 'Property:\\1')
  tmp <- lapply(x, function(page, ...){
    result <- WikipediR::page_content(
      domain = "wikidata.org",
      page_name = page,
      as_wikitext = TRUE,
      httr::user_agent("wikitaxa"),
      ...
    )
    out <- jsonlite::fromJSON(result$parse$wikitext[[1]])
    return(out)
  })

  list(
    property_value = tmp[[1]]$labels$en$value,
    property_description = tmp[[1]]$descriptions$en$value
  )
}

create_claims <- function(x) {
  lapply(x, function(z) {
    ff <- c(
      property = paste0(unique(z$mainsnak$property), collapse = ","),
      fetch_property(unique(z$mainsnak$property)),
      value = {
        if (inherits(z$mainsnak$datavalue$value, "data.frame")) {
          paste0(z$mainsnak$datavalue$value$`numeric-id`, collapse = ",")
        } else {
          paste0(z$mainsnak$datavalue$value, collapse = ",")
        }
      }
    )
    ff[vapply(ff, is.null, logical(1))] <- NA
    ff
  })
}

context("wt_data")

test_that("wt_data returns the correct class", {
  skip_on_cran()

  prop <- "P846"
  aa <- wt_data("Mimulus foliatus", property = prop)

  expect_is(aa, "list")
  expect_is(aa$labels, "data.frame")
  expect_is(aa$descriptions, "data.frame")
  expect_is(aa$aliases, "data.frame")
  expect_is(aa$sitelinks, "data.frame")
  expect_is(aa$claims, "data.frame")
  expect_is(aa$claims, "data.frame")
  expect_equal(aa$claims$property, prop)
})

test_that("wt_data fails well", {
  expect_error(wt_data(), "argument \"x\" is missing, with no default")
})

tc <- function(l) Filter(Negate(is.null), l)

dt_df <- function(x) {
  (ffff <- data.table::setDF(data.table::rbindlist(x, fill = TRUE,
                                                   use.names = TRUE)))
}

search_base <- function(x, y = "wikimedia") {
  sprintf("https://%s.%s.org/w/api.php", x, y)
}

atbl <- function(x) tibble::as_tibble(x)

g_et <- function(url, args = list(), ...) {
  cli <- crul::HttpClient$new(url = url)
  res <- cli$get(query = args, ...)
  res$raise_for_status()
  jsonlite::fromJSON(res$parse("UTF-8"))
}

assert <- function(x, y) {
  if (!is.null(x)) {
    if (!class(x) %in% y) {
      stop(deparse(substitute(x)), " must be of class ",
           paste0(y, collapse = ", "), call. = FALSE)
    }
  }
}

sh <- function(query, limit, offset, utf8) {
  assert(limit, c("integer", "numeric"))
  assert(offset, c("integer", "numeric"))
  assert(utf8, "logical")
  tc(list(
    action = "query", list = "search", srsearch = query,
    utf8 = if (utf8) "" else NULL, format = "json",
    srprop = "size|wordcount|timestamp|snippet",
    srlimit = limit, sroffset = offset
  ))
}

match_ <- function(string, pattern) {
  pos <- regexec(pattern, string)
  regmatches(string, pos)[[1]]
}

strex <- function(string, pattern) {
  regmatches(string, gregexpr(pattern, string))
}

#' Taxonomic Information from Wikipedia
#'
#' @name wikitaxa-package
#' @aliases wikitaxa
#' @docType package
#' @author Scott Chamberlain \email{myrmecocystus@@gmail.com}
#' @author Ethan Welty
#' @keywords package
NULL

#' List of Wikipedias
#'
#' data.frame of 295 rows, with 3 columns:
#' \itemize{
#'  \item language - language
#'  \item language_local - language in local name
#'  \item wiki - langugae code for the wiki
#' }
#'
#' From <https://meta.wikimedia.org/wiki/List_of_Wikipedias>
#'
#' @name wikipedias
#' @docType data
#' @keywords data
NULL

# MediaWiki (general) ----------------

#' Parse MediaWiki Page URL
#'
#' Parse a MediaWiki page url into its component parts (wiki name, wiki type,
#' and page title). Supports both static page urls and their equivalent API
#' calls.
#'
#' @export
#' @param url (character) MediaWiki page url.
#' @family MediaWiki functions
#' @return a list with elements:
#' \itemize{
#'  \item wiki - wiki language
#'  \item type - wikipedia type
#'  \item page - page name
#' }
#' @examples
#' wt_wiki_url_parse(url="https://en.wikipedia.org/wiki/Malus_domestica")
#' wt_wiki_url_parse("https://en.wikipedia.org/w/api.php?page=Malus_domestica")
wt_wiki_url_parse <- function(url) {
  url <- curl::curl_unescape(url)
  if (grepl("/w/api.php?", url)) {
    matches <-
      match_(
        url, "//([^\\.]+).([^\\.]+).[^/]*/w/api\\.php\\?.*page=([^&]+).*$")
  } else {
    matches <- match_(url, "//([^\\.]+).([^\\.]+).[^/]*/wiki/([^\\?]+)")
  }
  return(list(
    wiki = matches[2],
    type = matches[3],
    page = matches[4]
  ))
}

#' Build MediaWiki Page URL
#'
#' Builds a MediaWiki page url from its component parts (wiki name, wiki type,
#' and page title). Supports both static page urls and their equivalent API
#' calls.
#'
#' @export
#' @param wiki (character | list) Either the wiki name or a list with
#' `$wiki`, `$type`, and `$page` (the output of [wt_wiki_url_parse()]).
#' @param type (character) Wiki type.
#' @param page (character) Wiki page title.
#' @param api (boolean) Whether to return an API call or a static page url
#' (default). If `FALSE`, all following (API-only) arguments are ignored.
#' @param action (character) See <https://en.wikipedia.org/w/api.php>
#' for supported actions. This function currently only supports "parse".
#' @param redirects (boolean) If the requested page is set to a redirect,
#' resolve it.
#' @param format (character) See <https://en.wikipedia.org/w/api.php>
#' for supported output formats.
#' @param utf8 (boolean) If `TRUE`, encodes most (but not all) non-ASCII
#' characters as UTF-8 instead of replacing them with hexadecimal escape
#' sequences.
#' @param prop (character) Properties to retrieve, either as a character vector
#' or pipe-delimited string. See
#' <https://en.wikipedia.org/w/api.php?action=help&modules=parse> for
#' supported properties.
#' @family MediaWiki functions
#' @return a URL (character)
#' @examples
#' wt_wiki_url_build(wiki = "en", type = "wikipedia", page = "Malus domestica")
#' wt_wiki_url_build(
#'   wt_wiki_url_parse("https://en.wikipedia.org/wiki/Malus_domestica"))
#' wt_wiki_url_build("en", "wikipedia", "Malus domestica", api = TRUE)
wt_wiki_url_build <- function(wiki, type = NULL, page = NULL, api = FALSE,
                           action = "parse", redirects = TRUE, format = "json",
                           utf8 = TRUE,
                           prop = c("text", "langlinks", "categories",
                                    "links", "templates", "images",
                                    "externallinks", "sections", "revid",
                                    "displaytitle", "iwlinks", "properties")) {

  assert(utf8, "logical")
  if (is.null(type) && is.null(page)) {
    type <- wiki$type
    page <- wiki$page
    wiki <- wiki$wiki
  }
  page <- gsub(" ", "_", page)
  if (api) {
    base_url <- paste0("https://", wiki, ".", type, ".org/w/api.php")
    # To ensure it is removed
    if (!utf8) utf8 <- ""
    prop <- paste(prop, collapse = "|")
    query <- c(page = page, mget(c("action", "redirects", "format", "utf8",
                                   "prop")))
    query <- query[vapply(query, "!=", logical(1), "")]
    url <- crul::url_build(base_url, query = query)
    return(url)
  } else {
    return(paste0("https://", wiki, ".", type, ".org/wiki/", page))
  }
}

#' Get MediaWiki Page from API
#'
#' Supports both static page urls and their equivalent API calls.
#'
#' @export
#' @param url (character) MediaWiki page url.
#' @param ... Arguments passed to [wt_wiki_url_build()] if `url`
#' is a static page url.
#' @family MediaWiki functions
#' @return an `HttpResponse` response object from \pkg{crul}
#' @details If the URL given is for a human readable html page,
#' we convert it to equivalent API call - if URL is already an API call,
#' we just use that.
#' @examples \dontrun{
#' wt_wiki_page("https://en.wikipedia.org/wiki/Malus_domestica")
#' }
wt_wiki_page <- function(url, ...) {
  stopifnot(inherits(url, "character"))
  if (!grepl("/w/api.php?", url)) {
    url <- wt_wiki_url_build(wt_wiki_url_parse(url), api = TRUE)
  }
  cli <- crul::HttpClient$new(url = url)
  res <- cli$get(...)
  res$raise_for_status()
  return(res)
}

#' Parse MediaWiki Page
#'
#' Parses common properties from the result of a MediaWiki API page call.
#'
#' @export
#' @param page ([crul::HttpResponse]) Result of [wt_wiki_page()]
#' @param types (character) List of properties to parse.
#' @param tidy (logical). tidy output to data.frames when possible.
#' Default: `FALSE`
#' @family MediaWiki functions
#' @return a list
#' @details Available properties currently not parsed:
#' title, displaytitle, pageid, revid, redirects, text, categories,
#' links, templates, images, sections, properties, ...
#' @examples \dontrun{
#' pg <- wt_wiki_page("https://en.wikipedia.org/wiki/Malus_domestica")
#' wt_wiki_page_parse(pg)
#' }
wt_wiki_page_parse <- function(page, types = c("langlinks", "iwlinks",
                                            "externallinks"),
                               tidy = FALSE) {
  stopifnot(inherits(page, "HttpResponse"))
  result <- list()
  json <- jsonlite::fromJSON(rawToChar(page$content), tidy)
  if (is.null(json$parse)) {
    return(result)
  }
  ## Links to equivalent page in other languages
  if ("langlinks" %in% types) {
    result$langlinks <- if (tidy) {
      atbl(json$parse$langlinks)
    } else {
      vapply(json$parse$langlinks, "[[", "", "url")
    }
  }
  ## Other wiki links
  if ("iwlinks" %in% types) {
    result$iwlinks <- if (tidy) {
      atbl(json$parse$iwlinks$url)
    } else {
      vapply(json$parse$iwlinks, "[[", "", "url")
    }
  }
  ## Links to external resources
  if ("externallinks" %in% types) {
    result$externallinks <- json$parse$externallinks
  }
  ## Return
  return(result)
}

context("wt_wiki_page/wt_wiki_page_parse")

test_that("wt_wiki_page returns a response object", {
  skip_on_cran()

  vcr::use_cassette("wt_wiki_page", {
    url <- "https://en.wikipedia.org/wiki/Malus_domestica"
    result <- wt_wiki_page(url)
  }, preserve_exact_body_bytes = TRUE)
  
  expect_is(result, "HttpResponse")
})

test_that("wt_wiki_page_parse returns non-empty results", {
  skip_on_cran()

  vcr::use_cassette("wt_wiki_page_non_empty", {
    url <- "https://en.wikipedia.org/wiki/Malus_domestica"
    pg <- wt_wiki_page(url)
    types <- c("langlinks", "iwlinks", "externallinks")
    result <- wt_wiki_page_parse(pg, types = types)
  }, preserve_exact_body_bytes = TRUE)

  expect_is(result, "list")
  for (fieldname in types) {
    expect_is(result[fieldname], "list")
    expect_gt(length(result[fieldname]), 0)
  }
})

test_that("wt_wiki_page_parse returns non-empty results", {
  skip_on_cran()

  vcr::use_cassette("wt_wiki_page_non_empty_common_names", {
    url <- "https://en.wikipedia.org/wiki/Malus_domestica"
    pg <- wt_wiki_page(url)
    types <- c("common_names")
    result <- wt_wiki_page_parse(pg, types = types)
  }, preserve_exact_body_bytes = TRUE)
 
  expect_is(result, "list")
  for (fieldname in types) {
    expect_is(result[fieldname], "list")
    expect_gt(length(result[fieldname]), 0)
  }
})

context("wt_wiki_url_build")

test_that("wt_wiki_url_build correctly builds static page url", {
  skip_on_cran()

  url <- "https://en.wikipedia.org/wiki/Malus_domestica"
  result <- wt_wiki_url_build("en", "wikipedia", "Malus domestica")
  expect_equal(result, url)
})

test_that("wt_wiki_url_build correctly builds API page url", {
  skip_on_cran()

  url <- gsub("\n|\\s+", "", "https://en.wikipedia.org/w/api.php?page=
              Malus_domestica&action=parse&redirects=TRUE&format=json&
              utf8=TRUE&prop=text")
  result <- wt_wiki_url_build("en", "wikipedia", "Malus domestica",
                              api = TRUE, action = "parse", redirects = TRUE,
                              format = "json", utf8 = TRUE, prop = "text")
  expect_equal(result, url)
})

context("wt_wiki_url_parse")

test_that("wt_wiki_url_parse correctly parses static page url", {
  skip_on_cran()

  url <- "https://en.wikipedia.org/wiki/Malus_domestica"
  result <- wt_wiki_url_parse(url)
  expect_is(result, "list")
  expect_equal(result$wiki, "en")
  expect_equal(result$type, "wikipedia")
  expect_equal(result$page, "Malus_domestica")
})

test_that("wt_wiki_url_parse correctly parses API page url", {
  skip_on_cran()

  url <- "https://en.wikipedia.org/w/api.php?page=Malus_domestica"
  result <- wt_wiki_url_parse(url)
  expect_is(result, "list")
  expect_equal(result$wiki, "en")
  expect_equal(result$type, "wikipedia")
  expect_equal(result$page, "Malus_domestica")
})

#' WikiCommons
#'
#' @export
#' @template args
#' @family Wikicommons functions
#' @return `wt_wikicommons` returns a list, with slots:
#' \itemize{
#'  \item langlinks - language page links
#'  \item externallinks - external links
#'  \item common_names - a data.frame with `name` and `language` columns
#'  \item classification - a data.frame with `rank` and `name` columns
#' }
#'
#' `wt_wikicommons_parse` returns a list
#'
#' `wt_wikicommons_search` returns a list with slots for `continue` and
#' `query`, where `query` holds the results, with `query$search` slot with
#' the search results
#' @references <https://www.mediawiki.org/wiki/API:Search> for help on search
#' @examples \dontrun{
#' # high level
#' wt_wikicommons(name = "Malus domestica")
#' wt_wikicommons(name = "Pinus contorta")
#' wt_wikicommons(name = "Ursus americanus")
#' wt_wikicommons(name = "Balaenoptera musculus")
#' 
#' wt_wikicommons(name = "Category:Poeae")
#' wt_wikicommons(name = "Category:Pinaceae")
#'
#' # low level
#' pg <- wt_wiki_page("https://commons.wikimedia.org/wiki/Malus_domestica")
#' wt_wikicommons_parse(pg)
#'
#' # search wikicommons
#' wt_wikicommons_search(query = "Pinus")
#'
#' ## use search results to dig into pages
#' res <- wt_wikicommons_search(query = "Pinus")
#' lapply(res$query$search$title[1:3], wt_wikicommons)
#' }
wt_wikicommons <- function(name, utf8 = TRUE, ...) {
  assert(name, "character")
  stopifnot(length(name) == 1)
  prop <- c("langlinks", "externallinks", "common_names", "classification")
  res <- wt_wiki_url_build(
    wiki = "commons", type = "wikimedia", page = name,
    utf8 = utf8,
    prop = prop)
  pg <- wt_wiki_page(res, ...)
  wt_wikicommons_parse(pg, prop, tidy = TRUE)
}

#' @export
#' @rdname wt_wikicommons
wt_wikicommons_parse <- function(page, types = c("langlinks", "iwlinks",
                                          "externallinks", "common_names",
                                          "classification"),
                                 tidy = FALSE) {

  result <- wt_wiki_page_parse(page, types = types, tidy = tidy)
  json <- jsonlite::fromJSON(rawToChar(page$content), simplifyVector = FALSE)
  # if output is NULL
  if (is.null(json$parse)) {
    return(result)
  }
  # if page not found
  txt <- xml2::read_html(json$parse$text[[1]])
  html <- tryCatch(
      xml2::xml_find_all(txt, 
        "//div[contains(., \"Domain\") or contains(., \"Phylum\")]")[[2]], 
      error = function(e) e)
    if (inherits(html, "error")) return(list())
  ## Common names
  if ("common_names" %in% types) {
    vernacular_html <- xml2::xml_find_all(txt,
                                          xpath = "//bdi[@class='vernacular']")
    # XML formats:
    # <bdi class="vernacular" lang="en"><a href="">name</a></bdi>
    # <bdi class="vernacular" lang="en">name</bdi>
    ## Name formats:
    # name1 / name2
    # name1, name2
    # name (category)
    cnms <- lapply(vernacular_html, function(x) {
      attributes <- xml2::xml_attrs(x)
      language <- attributes[["lang"]]
      name <- trimws(gsub("[ ]*\\(.*\\)", "", xml2::xml_text(x)))
      list(
        name = name,
        language = language
      )
    })
    result$common_names <- if (tidy) atbl(dt_df(cnms)) else cnms
  }
  ## classification
  if ("classification" %in% types) {
    html <- tryCatch(
      xml2::xml_find_all(txt, 
        "//div[contains(., \"Domain\") or contains(., \"Phylum\")]")[[2]], 
      error = function(e) e)

    # labels
    labels <- c(gsub(":", "", strex(xml2::xml_text(html), "[A-Za-z]+\\)?:")[[1]]), "Authority")
    labels <- gsub("\\(|\\)", "", labels)
    labels <- labels[-1]

    # values    
    values <- xml2::xml_text(
      xml2::xml_find_all(if (inherits(html, "xml_nodes")) html[[2]] else html, ".//b"))
    values <- gsub("^:\\s+|^.+:\\s?", "", values)
    values <- values[-1]

    clz <- mapply(list, rank = labels, name = values,
                  SIMPLIFY = FALSE, USE.NAMES = FALSE)
    result$classification <- if (tidy) atbl(dt_df(clz)) else clz
  }
  return(result)
}

#' @export
#' @rdname wt_wikicommons
wt_wikicommons_search <- function(query, limit = 10, offset = 0, utf8 = TRUE,
                                  ...) {
  tmp <- g_et(search_base("commons"), sh(query, limit, offset, utf8), ...)
  tmp$query$search <- atbl(tmp$query$search)
  return(tmp)
}

#' Wikipedia
#'
#' @export
#' @template args
#' @param wiki (character) wiki language. default: en. See [wikipedias] for
#' language codes.
#' @family Wikipedia functions
#' @return `wt_wikipedia` returns a list, with slots:
#' \itemize{
#'  \item langlinks - language page links
#'  \item externallinks - external links
#'  \item common_names - a data.frame with `name` and `language` columns
#'  \item classification - a data.frame with `rank` and `name` columns
#'  \item synonyms - a character vector with taxonomic names
#' }
#'
#' `wt_wikipedia_parse` returns a list with same slots determined by
#' the `types` parmeter
#'
#' `wt_wikipedia_search` returns a list with slots for `continue` and
#' `query`, where `query` holds the results, with `query$search` slot with
#' the search results
#' @references <https://www.mediawiki.org/wiki/API:Search> for help on search
#' @examples \dontrun{
#' # high level
#' wt_wikipedia(name = "Malus domestica")
#' wt_wikipedia(name = "Malus domestica", wiki = "fr")
#' wt_wikipedia(name = "Malus domestica", wiki = "da")
#'
#' # low level
#' pg <- wt_wiki_page("https://en.wikipedia.org/wiki/Malus_domestica")
#' wt_wikipedia_parse(pg)
#' wt_wikipedia_parse(pg, tidy = TRUE)
#'
#' # search wikipedia
#' # FIXME: utf=FALSE for now until curl::curl_escape fix 
#' # https://github.com/jeroen/curl/issues/228
#' wt_wikipedia_search(query = "Pinus", utf8=FALSE)
#' wt_wikipedia_search(query = "Pinus", wiki = "fr", utf8=FALSE)
#' wt_wikipedia_search(query = "Pinus", wiki = "br", utf8=FALSE)
#'
#' ## curl options
#' # wt_wikipedia_search(query = "Pinus", verbose = TRUE, utf8=FALSE)
#'
#' ## use search results to dig into pages
#' res <- wt_wikipedia_search(query = "Pinus", utf8=FALSE)
#' lapply(res$query$search$title[1:3], wt_wikipedia)
#' }
wt_wikipedia <- function(name, wiki = "en", utf8 = TRUE, ...) {
  assert(name, "character")
  assert(wiki, "character")
  stopifnot(length(name) == 1)
  prop <- c("langlinks", "externallinks", "common_names", "classification",
            "synonyms")
  res <- wt_wiki_url_build(
    wiki = wiki, type = "wikipedia", page = name,
    utf8 = utf8,
    prop = prop)
  pg <- wt_wiki_page(res, ...)
  wt_wikipedia_parse(page = pg, types = prop, tidy = TRUE)
}

#' @export
#' @rdname wt_wikipedia
wt_wikipedia_parse <- function(page, types = c("langlinks", "iwlinks",
                                         "externallinks", "common_names",
                                         "classification"),
                               tidy = FALSE) {

  result <- wt_wiki_page_parse(page, types = types, tidy = tidy)
  json <- jsonlite::fromJSON(rawToChar(page$content), simplifyVector = TRUE)
  if (is.null(json$parse)) {
    return(result)
  }
  ## Common names
  if ("common_names" %in% types) {
    xml <- xml2::read_html(json$parse$text[[1]])
    names_xml <- list(
      regular_bolds = xml2::xml_find_all(
        xml,
        xpath = "/html/body/p[count(preceding::div[contains(@id, 'toc') or contains(@class, 'toc')]) = 0 and count(preceding::h1) = 0 and count(preceding::h2) = 0 and count(preceding::h3) = 0]//b[not(parent::*[self::i]) and not(i)]"), #nolint
      regular_biotabox_header =
        xml2::xml_find_all(
          xml,
          xpath = "(//table[contains(@class, 'infobox biota') or contains(@class, 'infobox_v2 biota')]//th)[1]/b[not(parent::*[self::i]) and not(i)]") #nolint
    )
    # NOTE: Often unreliable.
    regular_title <- stats::na.omit(
      match_(json$parse$displaytitle, "^([^<]*)$")[2])
    common_names <- unique(c(unlist(lapply(names_xml, xml2::xml_text)),
                             regular_title))
    language <- match_(page$url, 'http[s]*://([^\\.]*)\\.')[2]
    cnms <- lapply(common_names, function(name) {
      list(name = name, language = language)
    })
    result$common_names <- if (tidy) atbl(dt_df(cnms)) else cnms
  }
  ## classification
  if ("classification" %in% types) {
    txt <- xml2::read_html(json$parse$text[[1]])
    html <-
      xml2::xml_find_all(txt, "//table[@class=\"infobox biota\"]//span")
    labels <- xml2::xml_attr(html, "class")
    labels <- gsub("^\\s+|\\s$|\\(|\\)", "", labels)
    values <- gsub("^\\s+|\\s$", "", xml2::xml_text(html))
    clz <- mapply(list, rank = labels, name = values,
                  SIMPLIFY = FALSE, USE.NAMES = FALSE)
    result$classification <- if (tidy) atbl(dt_df(clz)) else clz
  }
  ## synonyms
  if ("synonyms" %in% types) {
    syns <- list()
    txt <- xml2::read_html(json$parse$text[[1]])
    html <-
      xml2::xml_find_all(txt, "//table[@class=\"infobox biota\"]//td")
    syn_node <-
      xml2::xml_find_first(html, "//th/a[contains(text(), \"Synonyms\")]")
    if (length(stats::na.omit(xml2::xml_text(syn_node))) > 0) {
      if (grepl("<br>", html[[length(html)]])) {
        syns <- xml2::xml_text(
          xml2::xml_find_all(
            xml2::xml_find_first(html[[length(html)]], "p"), "i"))
      } else {
        syn <- strsplit(xml2::xml_text(html[length(html)]), "\n")[[1]]
        syns <- syn[nzchar(syn)]
      }
    }
    result$synonyms <- syns
  }

  return(result)
}

#' @export
#' @rdname wt_wikipedia
wt_wikipedia_search <- function(query, wiki = "en", limit = 10, offset = 0,
                                utf8 = TRUE, ...) {

  assert(wiki, "character")
  tmp <- g_et(search_base(wiki, "wikipedia"), sh(query, limit, offset, utf8),
              ...)
  tmp$query$search <- atbl(tmp$query$search)
  return(tmp)
}

context("wt_wikipedia")

test_that("wt_wikipedia returns non-empty results", {
  skip_on_cran()

  aa <- wt_wikipedia(name = "Malus domestica")

  expect_is(aa, "list")
  expect_named(aa, c('langlinks', 'externallinks', 'common_names',
                     'classification', 'synonyms'))
  expect_is(aa$langlinks, "data.frame")
  expect_is(aa$externallinks, "character")
  expect_is(aa$common_names, "data.frame")
  expect_named(aa$common_names, c('name', 'language'))
  expect_is(aa$classification, "data.frame")
  expect_named(aa$classification, c('rank', 'name'))

  bb <- wt_wikipedia(name = "Poa annua")

  expect_is(bb, "list")
  expect_named(bb, c('langlinks', 'externallinks', 'common_names',
                     'classification', 'synonyms'))
  expect_is(bb$langlinks, "data.frame")
  expect_is(bb$externallinks, "character")
  expect_is(bb$common_names, "data.frame")
  expect_is(bb$classification, "data.frame")
  expect_named(bb$classification, c('rank', 'name'))
})

test_that("wt_wikipedia fails well", {
  skip_on_cran()

  expect_error(wt_wikipedia(),
            "argument \"name\" is missing")
  expect_error(wt_wikipedia(5),
               "name must be of class character")

  # "name" must be length 1
  expect_error(
    wt_wikipedia(c("Pinus", "asdfadsf")),
    "length\\(name\\) == 1 is not TRUE"
  )

  # "wiki" must be character
  expect_error(
    wt_wikipedia("Pinus", 5),
    "wiki must be of class character"
  )

  # "utf8" must be logical
  expect_error(
    wt_wikipedia("Pinus", utf8 = "asdf"),
    "utf8 must be of class logical"
  )
})

context("wt_wikipedia_parse")

test_that("wt_wikipedia_parse returns non-empty results", {
  skip_on_cran()

  url <- "https://species.wikimedia.org/wiki/Malus_domestica"
  pg <- wt_wiki_page(url)
  types <- c("common_names")
  result <- wt_wikipedia_parse(pg, types = types)
  expect_is(result, "list")
  for (fieldname in types) {
    expect_is(result[fieldname], "list")
    expect_gt(length(result[fieldname]), 0)
  }
})

context("wt_wikipedia_search")

test_that("wt_wikipedia_search works", {
  skip_on_cran()

  aa <- wt_wikipedia_search(query = "Pinus")

  expect_is(aa, "list")
  expect_is(aa$continue, "list")
  expect_is(aa$query, "list")
  expect_is(aa$query$searchinfo, "list")
  expect_is(aa$query$search, "data.frame")
  expect_named(aa$query$search, c('ns', 'title', 'pageid', 'size', 'wordcount',
                                  'snippet', 'timestamp'))

  # no results when not found
  expect_equal(NROW(wt_wikipedia_search("asdfadfaadfadfs")$query$search), 0)
})

test_that("wt_wikipedia_search fails well", {
  skip_on_cran()

  expect_error(
    wt_wikipedia_search(),
    "argument \"query\" is missing"
  )
  expect_error(
    wt_wikipedia_search("Pinus", limit = "adf"),
    "limit must be of class integer, numeric"
  )
  expect_error(
    wt_wikipedia_search("Pinus", offset = "adf"),
    "offset must be of class integer, numeric"
  )
})

#' WikiSpecies
#'
#' @export
#' @template args
#' @family Wikispecies functions
#' @return `wt_wikispecies` returns a list, with slots:
#' \itemize{
#'  \item langlinks - language page links
#'  \item externallinks - external links
#'  \item common_names - a data.frame with `name` and `language` columns
#'  \item classification - a data.frame with `rank` and `name` columns
#' }
#'
#' `wt_wikispecies_parse` returns a list
#'
#' `wt_wikispecies_search` returns a list with slots for `continue` and
#' `query`, where `query` holds the results, with `query$search` slot with
#' the search results
#' @references <https://www.mediawiki.org/wiki/API:Search> for help on search
#' @examples \dontrun{
#' # high level
#' wt_wikispecies(name = "Malus domestica")
#' wt_wikispecies(name = "Pinus contorta")
#' wt_wikispecies(name = "Ursus americanus")
#' wt_wikispecies(name = "Balaenoptera musculus")
#'
#' # low level
#' pg <- wt_wiki_page("https://species.wikimedia.org/wiki/Abelmoschus")
#' wt_wikispecies_parse(pg)
#'
#' # search wikispecies
#' wt_wikispecies_search(query = "pine tree")
#'
#' ## use search results to dig into pages
#' res <- wt_wikispecies_search(query = "pine tree")
#' lapply(res$query$search$title[1:3], wt_wikispecies)
#' }
wt_wikispecies <- function(name, utf8 = TRUE, ...) {
  assert(name, "character")
  stopifnot(length(name) == 1)
  prop <- c("langlinks", "externallinks", "common_names", "classification")
  res <- wt_wiki_url_build(
    wiki = "species", type = "wikimedia", page = name,
    utf8 = utf8,
    prop = prop)
  pg <- wt_wiki_page(res, ...)
  wt_wikispecies_parse(pg, prop, tidy = TRUE)
}

#' @export
#' @rdname wt_wikispecies
wt_wikispecies_parse <- function(page, types = c("langlinks", "iwlinks",
                                              "externallinks", "common_names",
                                                 "classification"),
                                 tidy = FALSE) {

  result <- wt_wiki_page_parse(page, types = types, tidy = tidy)
  json <- jsonlite::fromJSON(rawToChar(page$content), simplifyVector = FALSE)
  if (is.null(json$parse)) {
    return(result)
  }
  ## Common names
  if ("common_names" %in% types) {
    xml <- xml2::read_html(json$parse$text[[1]])
    # XML formats:
    # <b>language:</b>&nbsp;[name|<a>name</a>]
    # Name formats:
    # name1, name2
    vernacular_html <- xml2::xml_find_all(
      xml,
      "(//h2/span[contains(@id, 'Vernacular')]/parent::*/following-sibling::div)[1]" #nolint
    )
    languages_html <- xml2::xml_find_all(vernacular_html, xpath = "b")
    languages <- gsub("\\s*:\\s*", "",
                      unlist(lapply(languages_html, xml2::xml_text)))
    names_html <-
      xml2::xml_find_all(
        vernacular_html,
    "b[not(following-sibling::*[1][self::a])]/following-sibling::text()[1] | b/following-sibling::*[1][self::a]/text()") #nolint
    common_names <- gsub("^\\s*", "",
                         unlist(lapply(names_html, xml2::xml_text)))
    cnms <-
      mapply(list, name = common_names,
             language = languages, SIMPLIFY = FALSE, USE.NAMES = FALSE)
    result$common_names <- if (tidy) atbl(dt_df(cnms)) else cnms
  }
  ## classification
  if ("classification" %in% types) {
    txt <- xml2::read_html(json$parse$text[[1]])
    html <- xml2::xml_text(
      xml2::xml_find_first(txt, "//table[contains(@class, \"wikitable\")]//p"))
    html <- strsplit(html, "\n")[[1]]
    labels <-
      vapply(html, function(z) strsplit(z, ":")[[1]][1], "", USE.NAMES = FALSE)
    values <-
      vapply(html, function(z) strsplit(z, ":")[[1]][2], "", USE.NAMES = FALSE)
    values <- gsub("^\\s+|\\s+$", "", values)
    clz <- mapply(list, rank = labels, name = values,
                  SIMPLIFY = FALSE, USE.NAMES = FALSE)
    result$classification <- if (tidy) atbl(dt_df(clz)) else clz
  }
  return(result)
}

#' @export
#' @rdname wt_wikispecies
wt_wikispecies_search <- function(query, limit = 10, offset = 0, utf8 = TRUE,
                                  ...) {
  tmp <- g_et(search_base("species"), sh(query, limit, offset, utf8), ...)
  tmp$query$search <- atbl(tmp$query$search)
  return(tmp)
}

context("wt_wikispecies")

test_that("wt_wikispecies returns non-empty results", {
  skip_on_cran()

  vcr::use_cassette("wt_wikispecies_malus", {
    aa <- wt_wikispecies(name = "Malus domestica")

    expect_is(aa, "list")
    expect_named(aa, c('langlinks', 'externallinks', 'common_names',
                       'classification'))
    expect_is(aa$langlinks, "data.frame")
    expect_is(aa$externallinks, "character")
    expect_is(aa$common_names, "data.frame")
    expect_named(aa$common_names, c('name', 'language'))
    expect_is(aa$classification, "data.frame")
    expect_named(aa$classification, c('rank', 'name'))
  })

  vcr::use_cassette("wt_wikispecies_poa", {
    bb <- wt_wikispecies(name = "Poa annua")

    expect_is(bb, "list")
    expect_named(bb, c('langlinks', 'externallinks', 'common_names',
                       'classification'))
    expect_is(bb$langlinks, "data.frame")
    expect_is(bb$externallinks, "character")
    expect_is(bb$common_names, "data.frame")
    expect_named(bb$common_names, c('name', 'language'))
    expect_is(bb$classification, "data.frame")
    expect_named(bb$classification, c('rank', 'name'))
  })
})

test_that("wt_wikispecies fails well", {
  skip_on_cran()

  expect_error(wt_wikispecies(),
            "argument \"name\" is missing")
  expect_error(wt_wikispecies(5),
               "name must be of class character")

  # "name" must be length 1
  expect_error(
    wt_wikispecies(c("Pinus", "asdfadsf")),
    "length\\(name\\) == 1 is not TRUE"
  )

  # "utf8" must be logical
  expect_error(
    wt_wikispecies("Pinus", "asdf"),
    "utf8 must be of class logical"
  )
})

context("wt_wikispecies_parse")

test_that("wt_wikispecies_parse returns non-empty results", {
  skip_on_cran()

  vcr::use_cassette("wt_wikispecies_parse", {
    url <- "https://species.wikimedia.org/wiki/Malus_domestica"
    pg <- wt_wiki_page(url)
    types <- c("common_names")
    result <- wt_wikispecies_parse(pg, types = types)
    expect_is(result, "list")
    for (fieldname in types) {
      expect_is(result[fieldname], "list")
      expect_gt(length(result[fieldname]), 0)
    }
  })
})

context("wt_wikispecies_search")

test_that("wt_wikispecies_search works", {
  skip_on_cran()

  vcr::use_cassette("wt_wikispecies_search", {
    aa <- wt_wikispecies_search(query = "Pinus")

    expect_is(aa, "list")
    expect_is(aa$continue, "list")
    expect_is(aa$query, "list")
    expect_is(aa$query$searchinfo, "list")
    expect_is(aa$query$search, "data.frame")
    expect_named(aa$query$search, c('ns', 'title', 'pageid', 'size', 'wordcount',
                                    'snippet', 'timestamp'))
  })

  # no results when not found
  vcr::use_cassette("wt_wikispecies_search_not_found", {
    expect_equal(NROW(wt_wikispecies_search("asdfadfaadfadfs")$query$search), 0)
  })
})

test_that("wt_wikispecies_search fails well", {
  skip_on_cran()

  expect_error(
    wt_wikispecies_search(),
    "argument \"query\" is missing"
  )
  expect_error(
    wt_wikispecies_search("Pinus", limit = "adf"),
    "limit must be of class integer, numeric"
  )
  expect_error(
    wt_wikispecies_search("Pinus", offset = "adf"),
    "offset must be of class integer, numeric"
  )
})

---
title: "Introduction to the wikitaxa package"
author: "Scott Chamberlain"
date: "2020-06-28"
output: 
    html_document:
        toc: true
        toc_float: true
        theme: readable
vignette: >
  %\VignetteIndexEntry{Introduction to the wikitaxa package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



`wikitaxa` - Taxonomy data from Wikipedia

The goal of `wikitaxa` is to allow search and taxonomic data retrieval from
across many Wikimedia sites, including: Wikipedia, Wikicommons, and
Wikispecies.

There are lower level and higher level parts to the package API:

### Low level API

The low level API is meant for power users and gives you more control,
but requires more knowledge.

* `wt_wiki_page()`
* `wt_wiki_page_parse()`
* `wt_wiki_url_build()`
* `wt_wiki_url_parse()`
* `wt_wikispecies_parse()`
* `wt_wikicommons_parse()`
* `wt_wikipedia_parse()`

### High level API

The high level API is meant to be easier and faster to use.

* `wt_data()`
* `wt_data_id()`
* `wt_wikispecies()`
* `wt_wikicommons()`
* `wt_wikipedia()`

Search functions:

* `wt_wikicommons_search()`
* `wt_wikispecies_search()`
* `wt_wikipedia_search()`

## Installation

CRAN version


```r
install.packages("wikitaxa")
```

Dev version


```r
remotes::install_github("ropensci/wikitaxa")
```


```r
library("wikitaxa")
```

## wiki data


```r
z <- wt_data("Poa annua")
names(z)
#> [1] "labels"       "descriptions" "aliases"      "sitelinks"    "claims"
head(z$labels)
#>   language            value
#> 1       pt        Poa annua
#> 2       is   Varpasveifgras
#> 3       pl Wiechlina roczna
#> 4       fr   Pâturin annuel
#> 5       es        Poa annua
#> 6       en        Poa annua
```

Get a Wikidata ID


```r
wt_data_id("Mimulus foliatus")
#> [1] "Q6495130"
#> attr(,"class")
#> [1] "wiki_id"
```

## wikipedia

lower level


```r
pg <- wt_wiki_page("https://en.wikipedia.org/wiki/Malus_domestica")
res <- wt_wiki_page_parse(pg)
res$iwlinks
#> [1] "https://commons.wikimedia.org/wiki/Category:Apples"         
#> [2] "https://commons.wikimedia.org/wiki/Category:Apple_cultivars"
#> [3] "https://www.wikidata.org/wiki/Q158657"                      
#> [4] "https://www.wikidata.org/wiki/Q18674606"                    
#> [5] "https://species.wikimedia.org/wiki/Malus_pumila"            
#> [6] "https://species.wikimedia.org/wiki/Malus_domestica"
```

higher level


```r
res <- wt_wikipedia("Malus domestica")
res$common_names
#> # A tibble: 1 x 2
#>   name  language
#>   <chr> <chr>   
#> 1 Apple en
res$classification
#> # A tibble: 3 x 2
#>   rank       name             
#>   <chr>      <chr>            
#> 1 plainlinks ""               
#> 2 binomial   "Malus domestica"
#> 3 <NA>       ""
```

choose a wikipedia language


```r
# French
wt_wikipedia(name = "Malus domestica", wiki = "fr")
#> $langlinks
#> # A tibble: 60 x 5
#>    lang   url                                   langname   autonym    `*`       
#>    <chr>  <chr>                                 <chr>      <chr>      <chr>     
#>  1 als    https://als.wikipedia.org/wiki/Kultu… Alemannis… Alemannis… Kulturapf…
#>  2 am     https://am.wikipedia.org/wiki/%E1%89… amharique  አማርኛ       ቱፋሕ       
#>  3 ast    https://ast.wikipedia.org/wiki/Malus… asturien   asturianu  Malus dom…
#>  4 az     https://az.wikipedia.org/wiki/M%C9%9… azéri      azərbayca… Mədəni al…
#>  5 bat-s… https://bat-smg.wikipedia.org/wiki/V… Samogitian žemaitėška Vuobelės  
#>  6 bg     https://bg.wikipedia.org/wiki/%D0%94… bulgare    български  Домашна я…
#>  7 bpy    https://bpy.wikipedia.org/wiki/%E0%A… bishnupri… বিষ্ণুপ্র…    আপেল      
#>  8 ca     https://ca.wikipedia.org/wiki/Pomera… catalan    català     Pomera co…
#>  9 cs     https://cs.wikipedia.org/wiki/Jablo%… tchèque    čeština    Jabloň do…
#> 10 csb    https://csb.wikipedia.org/wiki/Dom%C… kachoube   kaszëbsczi Domôcô ja…
#> # … with 50 more rows
#> 
#> $externallinks
#>  [1] "http://www.cabi-publishing.org/pdf/Books/0851995926/0851995926_Chap01.pdf"                 
#>  [2] "http://www.umass.edu/fruitadvisor/fruitnotes/ontheorigin.pdf"                              
#>  [3] "http://www.applegenome.org"                                                                
#>  [4] "http://societeradio-canada.info/emissions/les_annees_lumiere/2010-2011/"                   
#>  [5] "http://www.nature.com/ng/journal/vaop/ncurrent/full/ng.654.html"                           
#>  [6] "https://gallica.bnf.fr/ark:/12148/bpt6k28582v"                                             
#>  [7] "http://worldcat.org/issn/1471-2229&lang=fr"                                                
#>  [8] "https://www.ncbi.nlm.nih.gov/pubmed/26924309"                                              
#>  [9] "https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4770685"                                      
#> [10] "https://dx.doi.org/10.1186%2Fs12870-016-0739-y"                                            
#> [11] "https://doi.org/10.1186/s12870-016-0739-y"                                                 
#> [12] "http://www.hamblenne.be/LISTE_RGF.pdf"                                                     
#> [13] "http://www.arcticapples.com/blog/john/demystifying-arctic-apples#.UaeX-Jzjmw5"             
#> [14] "http://www.cctec.cornell.edu/plants/GENEVA-Apple-Rootstocks-Comparison-Chart-120911.pdf"   
#> [15] "https://commons.wikimedia.org/wiki/Category:Malus_domestica?uselang=fr"                    
#> [16] "http://www.tela-botanica.org/page:eflore"                                                  
#> [17] "http://www.tela-botanica.org/bdtfx-nn-40744"                                               
#> [18] "http://www.cbif.gc.ca/acp/fra/siti/regarder?tsn=516655"                                    
#> [19] "http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=516655"      
#> [20] "http://www.cbif.gc.ca/acp/fra/siti/regarder?tsn=25262"                                     
#> [21] "http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=25262"       
#> [22] "http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?lin=s&p=has_linkout&id=3750"       
#> [23] "http://www.ars-grin.gov/"                                                                  
#> [24] "https://npgsweb.ars-grin.gov/gringlobal/taxonomydetail.aspx?104681"                        
#> [25] "https://www.biolib.cz/en/"                                                                 
#> [26] "https://www.biolib.cz/en/taxon/id39552/"                                                   
#> [27] "http://inpn.mnhn.fr/isb/espece/cd_nom/107207"                                              
#> [28] "http://www.bmlisieux.com/normandie/roblet.htm"                                             
#> [29] "http://site.voila.fr/babadubonsai/docum/docmal.html"                                       
#> [30] "http://www.fruitiers.net"                                                                  
#> [31] "http://www.inra.fr/hyppz/CULTURES/3c---003.htm"                                            
#> [32] "http://cat.inist.fr/?aModele=afficheN&cpsidt=15506238"                                     
#> [33] "http://www.omafra.gov.on.ca/french/crops/facts/98-014.htm"                                 
#> [34] "http://www.gardenaction.co.uk/fruit_veg_diary/fruit_veg_mini_project_september_2_apple.asp"
#> 
#> $common_names
#> # A tibble: 1 x 2
#>   name               language
#>   <chr>              <chr>   
#> 1 Pommier domestique fr      
#> 
#> $classification
#> # A tibble: 0 x 0
#> 
#> $synonyms
#> list()
# Slovak
wt_wikipedia(name = "Malus domestica", wiki = "sk")
#> $langlinks
#> # A tibble: 60 x 5
#>    lang   url                               langname        autonym    `*`      
#>    <chr>  <chr>                             <chr>           <chr>      <chr>    
#>  1 als    https://als.wikipedia.org/wiki/K… Alemannisch     Alemannis… Kulturap…
#>  2 am     https://am.wikipedia.org/wiki/%E… amharčina       አማርኛ       ቱፋሕ      
#>  3 ast    https://ast.wikipedia.org/wiki/M… astúrčina       asturianu  Malus do…
#>  4 az     https://az.wikipedia.org/wiki/M%… azerbajdžančina azərbayca… Mədəni a…
#>  5 bat-s… https://bat-smg.wikipedia.org/wi… Samogitian      žemaitėška Vuobelės 
#>  6 bg     https://bg.wikipedia.org/wiki/%D… bulharčina      български  Домашна …
#>  7 bpy    https://bpy.wikipedia.org/wiki/%… bišnuprijskoma… বিষ্ণুপ্র…    আপেল     
#>  8 ca     https://ca.wikipedia.org/wiki/Po… katalánčina     català     Pomera c…
#>  9 cs     https://cs.wikipedia.org/wiki/Ja… čeština         čeština    Jabloň d…
#> 10 csb    https://csb.wikipedia.org/wiki/D… kašubčina       kaszëbsczi Domôcô j…
#> # … with 50 more rows
#> 
#> $externallinks
#> list()
#> 
#> $common_names
#> # A tibble: 1 x 2
#>   name          language
#>   <chr>         <chr>   
#> 1 Jabloň domáca sk      
#> 
#> $classification
#> # A tibble: 0 x 0
#> 
#> $synonyms
#> list()
# Vietnamese
wt_wikipedia(name = "Malus domestica", wiki = "vi")
#> $langlinks
#> # A tibble: 60 x 5
#>    lang     url                                 langname    autonym   `*`       
#>    <chr>    <chr>                               <chr>       <chr>     <chr>     
#>  1 als      https://als.wikipedia.org/wiki/Kul… Alemannisch Alemanni… Kulturapf…
#>  2 am       https://am.wikipedia.org/wiki/%E1%… Tiếng Amha… አማርኛ      ቱፋሕ       
#>  3 ast      https://ast.wikipedia.org/wiki/Mal… Tiếng Astu… asturianu Malus dom…
#>  4 az       https://az.wikipedia.org/wiki/M%C9… Tiếng Azer… azərbayc… Mədəni al…
#>  5 zh-min-… https://zh-min-nan.wikipedia.org/w… Chinese (M… Bân-lâm-… Phōng-kó-…
#>  6 bg       https://bg.wikipedia.org/wiki/%D0%… Tiếng Bulg… български Домашна я…
#>  7 ca       https://ca.wikipedia.org/wiki/Pome… Tiếng Cata… català    Pomera co…
#>  8 cs       https://cs.wikipedia.org/wiki/Jabl… Tiếng Séc   čeština   Jabloň do…
#>  9 da       https://da.wikipedia.org/wiki/Almi… Tiếng Đan … dansk     Almindeli…
#> 10 de       https://de.wikipedia.org/wiki/Kult… Tiếng Đức   Deutsch   Kulturapf…
#> # … with 50 more rows
#> 
#> $externallinks
#>  [1] "http://biology.umaine.edu/Amelanchier/Rosaceae_2007.pdf"                                       
#>  [2] "//dx.doi.org/10.1007%2Fs00606-007-0539-9"                                                      
#>  [3] "https://npgsweb.ars-grin.gov/gringlobal/taxonomydetail.aspx?410495"                            
#>  [4] "https://npgsweb.ars-grin.gov/gringlobal/taxonomydetail.aspx?30530"                             
#>  [5] "http://www.uga.edu/fruit/apple.html"                                                           
#>  [6] "//dx.doi.org/10.3732%2Fajb.93.3.357"                                                           
#>  [7] "http://www.plosgenetics.org/article/info:doi%2F10.1371%2Fjournal.pgen.1002703"                 
#>  [8] "//www.ncbi.nlm.nih.gov/pmc/articles/PMC3349737"                                                
#>  [9] "//www.ncbi.nlm.nih.gov/pubmed/22589740"                                                        
#> [10] "//dx.doi.org/10.1371%2Fjournal.pgen.1002703"                                                   
#> [11] "http://news.sciencemag.org/sciencenow/2012/05/scienceshot-the-secret-history-o.html"           
#> [12] "http://www.plantpress.com/wildlife/o523-apple.php"                                             
#> [13] "http://cahnrsnews.wsu.edu/2010/08/29/apple-cup-rivals-contribute-to-apple-genome-sequencing/"  
#> [14] "http://www.nature.com/ng/journal/v42/n10/full/ng.654.html"                                     
#> [15] "http://www.alphagalileo.org/ViewItem.aspx?ItemId=83717&CultureCode=en"                         
#> [16] "http://www.ornl.gov/sci/techresources/Human_Genome/project/info.shtml"                         
#> [17] "https://commons.wikimedia.org/wiki/Apple?uselang=vi"                                           
#> [18] "https://commons.wikimedia.org/wiki/Category:Malus_domestica?uselang=vi"                        
#> [19] "http://bachkhoatoanthu.vass.gov.vn/noidung/tudien/Lists/GiaiNghia/View_Detail.aspx?ItemID=5007"
#> [20] "http://www.eol.org/pages/629094"                                                               
#> [21] "http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?id=3750"                               
#> [22] "http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=516655"          
#> [23] "http://www.catalogueoflife.org/col/details/species/id/19538828/synonym/19539435"               
#> [24] "https://www.biolib.cz/cz/taxon/id39552"                                                        
#> [25] "https://gd.eppo.int/taxon/MABSD"                                                               
#> [26] "https://npgsweb.ars-grin.gov/gringlobal/taxonomydetail.aspx?id=104681"                         
#> [27] "http://www.ipni.org/ipni/idPlantNameSearch.do?id=726282-1"                                     
#> [28] "https://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=516655"         
#> [29] "https://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?mode=Info&id=3750"                    
#> [30] "http://www.nzor.org.nz/names/14d024a2-d821-48e3-95d8-f0dd206c70a0"                             
#> [31] "http://www.pfaf.org/user/Plant.aspx?LatinName=Malus+domestica"                                 
#> [32] "http://www.theplantlist.org/tpl1.1/record/rjp-454"                                             
#> [33] "http://www.plantsoftheworldonline.org/taxon/urn:lsid:ipni.org:names:726282-1"                  
#> [34] "http://legacy.tropicos.org/Name/27804420"                                                      
#> [35] "https://vicflora.rbg.vic.gov.au/flora/taxon/e41b929d-b709-4f4c-8dbe-2a9241e2342b"              
#> [36] "http://www.ipni.org/ipni/idPlantNameSearch.do?id=60476301-2"                                   
#> [37] "http://www.plantsoftheworldonline.org/taxon/urn:lsid:ipni.org:names:60476301-2"                
#> [38] "http://legacy.tropicos.org/Name/100473089"                                                     
#> 
#> $common_names
#> # A tibble: 1 x 2
#>   name            language
#>   <chr>           <chr>   
#> 1 Malus domestica vi      
#> 
#> $classification
#> # A tibble: 0 x 0
#> 
#> $synonyms
#> list()
```

search


```r
wt_wikipedia_search(query = "Pinus")
#> $batchcomplete
#> [1] ""
#> 
#> $continue
#> $continue$sroffset
#> [1] 10
#> 
#> $continue$continue
#> [1] "-||"
#> 
#> 
#> $query
#> $query$searchinfo
#> $query$searchinfo$totalhits
#> [1] 3374
#> 
#> $query$searchinfo$suggestion
#> [1] "penis"
#> 
#> $query$searchinfo$suggestionsnippet
#> [1] "penis"
#> 
#> 
#> $query$search
#> # A tibble: 10 x 7
#>       ns title     pageid  size wordcount snippet                    timestamp  
#>    <int> <chr>      <int> <int>     <int> <chr>                      <chr>      
#>  1     0 Pine       39389 36555      4058 "A pine is any conifer in… 2020-06-26…
#>  2     0 Pinus po… 532941 31087      3069 "misidentified it as <spa… 2020-06-19…
#>  3     0 Pinus co… 507717 20486      2343 "all pines (member specie… 2020-05-25…
#>  4     0 Pinus je… 463015  9130      1008 "long, with a large (15 t… 2019-12-22…
#>  5     0 Pinus st… 464301 31478      3815 "3 ft) tall &amp; wide. M… 2020-06-22…
#>  6     0 Pinus re… 507802  7501       783 "&quot;<span class=\"sear… 2020-05-08…
#>  7     0 Pinus lo… 649634 15408      1741 "sometimes form dense for… 2020-04-14…
#>  8     0 Pinus la… 459402 11464      1338 "Fire affected this speci… 2020-01-15…
#>  9     0 Pinus ni… 438963 11947      1421 "hypodermal cells. P. nig… 2020-04-03…
#> 10     0 Pinus mu… 438946 11964       901 "encyclopedia) is still r… 2020-06-17…
```

search supports languages


```r
wt_wikipedia_search(query = "Pinus", wiki = "fr")
#> $batchcomplete
#> [1] ""
#> 
#> $continue
#> $continue$sroffset
#> [1] 10
#> 
#> $continue$continue
#> [1] "-||"
#> 
#> 
#> $query
#> $query$searchinfo
#> $query$searchinfo$totalhits
#> [1] 990
#> 
#> 
#> $query$search
#> # A tibble: 10 x 7
#>       ns title     pageid  size wordcount snippet                    timestamp  
#>    <int> <chr>      <int> <int>     <int> <chr>                      <chr>      
#>  1     0 Pin (pl…   89798 83647      9325 "<span class=\"searchmatc… 2020-05-23…
#>  2     0 Pinus p…  121544 31274      3892 "<span class=\"searchmatc… 2020-04-30…
#>  3     0 Pinus c…   98421  8237       959 "<span class=\"searchmatc… 2019-05-30…
#>  4     0 Pinus n…  950330 26623      3013 "recycler}}. <span class=… 2020-03-30…
#>  5     0 Pin syl…  121562 13725      1611 "<span class=\"searchmatc… 2020-03-22…
#>  6     0 Pinus h…  117280 22257      2671 "<span class=\"searchmatc… 2020-05-07…
#>  7     0 Pin par…  138378  8763       916 "<span class=\"searchmatc… 2020-04-26…
#>  8     0 Pinus s…  776950 11662      1628 "les articles homonymes, … 2020-04-14…
#>  9     0 Pinus m… 2480854 21747      2310 "<span class=\"searchmatc… 2019-02-25…
#> 10     0 Pinus u… 3208429  6316       720 "significations, voir Pin… 2020-03-09…
```


## wikicommons

lower level


```r
pg <- wt_wiki_page("https://commons.wikimedia.org/wiki/Abelmoschus")
res <- wt_wikicommons_parse(pg)
res$common_names[1:3]
#> [[1]]
#> [[1]]$name
#> [1] "okra"
#> 
#> [[1]]$language
#> [1] "en"
#> 
#> 
#> [[2]]
#> [[2]]$name
#> [1] "مسكي"
#> 
#> [[2]]$language
#> [1] "ar"
#> 
#> 
#> [[3]]
#> [[3]]$name
#> [1] "Abelmoş"
#> 
#> [[3]]$language
#> [1] "az"
```

higher level


```r
res <- wt_wikicommons("Abelmoschus")
res$classification
#> # A tibble: 15 x 2
#>    rank       name            
#>    <chr>      <chr>           
#>  1 Domain     "Eukaryota"     
#>  2 unranked   "Archaeplastida"
#>  3 Regnum     "Plantae"       
#>  4 Cladus     "angiosperms"   
#>  5 Cladus     "eudicots"      
#>  6 Cladus     "core eudicots" 
#>  7 Cladus     "superrosids"   
#>  8 Cladus     "rosids"        
#>  9 Cladus     "eurosids II"   
#> 10 Ordo       "Malvales"      
#> 11 Familia    "Malvaceae"     
#> 12 Subfamilia "Malvoideae"    
#> 13 Tribus     "Hibisceae"     
#> 14 Genus      "Abelmoschus"   
#> 15 Authority  " Medik. (1787)"
res$common_names
#> # A tibble: 19 x 2
#>    name             language
#>    <chr>            <chr>   
#>  1 okra             en      
#>  2 مسكي             ar      
#>  3 Abelmoş          az      
#>  4 Bamja            bs      
#>  5 Ibiškovec        cs      
#>  6 Bisameibisch     de      
#>  7 Okrat            fi      
#>  8 Abelmosco        gl      
#>  9 Abelmošus        hr      
#> 10 Ybiškė           lt      
#> 11 അബെൽമോസ്കസ്        ml      
#> 12 Абельмош         mrj     
#> 13 Abelmoskusslekta nn      
#> 14 Piżmian          pl      
#> 15 Абельмош         ru      
#> 16 Okrasläktet      sv      
#> 17 Абельмош         udm     
#> 18 Chi Vông vang    vi      
#> 19 黄葵属           zh
```

search


```r
wt_wikicommons_search(query = "Pinus")
#> $batchcomplete
#> [1] ""
#> 
#> $continue
#> $continue$sroffset
#> [1] 10
#> 
#> $continue$continue
#> [1] "-||"
#> 
#> 
#> $query
#> $query$searchinfo
#> $query$searchinfo$totalhits
#> [1] 270
#> 
#> 
#> $query$search
#> # A tibble: 10 x 7
#>       ns title                    pageid size  wordcount snippet timestamp      
#>    <int> <chr>                     <int> <lgl>     <int> <chr>   <chr>          
#>  1     0 Pinus sylvestris           9066 NA            0 ""      2020-05-13T19:…
#>  2     0 Pinus ponderosa          250435 NA            0 ""      2020-04-18T15:…
#>  3     0 Pinus nigra               64703 NA            0 ""      2018-03-06T10:…
#>  4     0 Pinus                     82071 NA            0 ""      2017-05-28T10:…
#>  5     0 Pinus mugo               132442 NA            0 ""      2019-07-26T09:…
#>  6     0 Rogów Arboretum        10563490 NA            0 ""      2020-01-01T13:…
#>  7     0 Pinus contorta           186918 NA            0 ""      2020-01-19T19:…
#>  8     0 Anacortes Community F…  2989013 NA            0 ""      2014-12-10T15:…
#>  9     0 Pinus halepensis         172181 NA            0 ""      2018-05-05T10:…
#> 10     0 Pinus brutia             139389 NA            0 ""      2014-11-23T11:…
```


## wikispecies

lower level


```r
pg <- wt_wiki_page("https://species.wikimedia.org/wiki/Malus_domestica")
res <- wt_wikispecies_parse(pg, types = "common_names")
res$common_names[1:3]
#> [[1]]
#> [[1]]$name
#> [1] "Ябълка"
#> 
#> [[1]]$language
#> [1] "български"
#> 
#> 
#> [[2]]
#> [[2]]$name
#> [1] "Poma, pomera"
#> 
#> [[2]]$language
#> [1] "català"
#> 
#> 
#> [[3]]
#> [[3]]$name
#> [1] "jabloň domácí"
#> 
#> [[3]]$language
#> [1] "čeština"
```

higher level


```r
res <- wt_wikispecies("Malus domestica")
res$classification
#> # A tibble: 8 x 2
#>   rank        name         
#>   <chr>       <chr>        
#> 1 Superregnum Eukaryota    
#> 2 Regnum      Plantae      
#> 3 Cladus      Angiosperms  
#> 4 Cladus      Eudicots     
#> 5 Cladus      Core eudicots
#> 6 Cladus      Rosids       
#> 7 Cladus      Eurosids I   
#> 8 Ordo        Rosales
res$common_names
#> # A tibble: 22 x 2
#>    name          language  
#>    <chr>         <chr>     
#>  1 Ябълка        български 
#>  2 Poma, pomera  català    
#>  3 jabloň domácí čeština   
#>  4 Apfel         Deutsch   
#>  5 Μηλιά         Ελληνικά  
#>  6 Apple         English   
#>  7 Manzano       español   
#>  8 Aed-õunapuu   eesti     
#>  9 Tarhaomenapuu suomi     
#> 10 Aapel         Nordfriisk
#> # … with 12 more rows
```

search


```r
wt_wikispecies_search(query = "Pinus")
#> $batchcomplete
#> [1] ""
#> 
#> $continue
#> $continue$sroffset
#> [1] 10
#> 
#> $continue$continue
#> [1] "-||"
#> 
#> 
#> $query
#> $query$searchinfo
#> $query$searchinfo$totalhits
#> [1] 515
#> 
#> 
#> $query$search
#> # A tibble: 10 x 7
#>       ns title         pageid  size wordcount snippet                timestamp  
#>    <int> <chr>          <int> <int>     <int> <chr>                  <chr>      
#>  1     0 Pinus         1.74e4  5737       784 "Familia: Pinaceae Ge… 2020-06-06…
#>  2     0 Pinus halepe… 4.51e4  4047       580 "Pinaceae Genus: <spa… 2019-12-20…
#>  3     0 Pinus pinea   4.51e4  1949       406 "Familia: Pinaceae Ge… 2019-10-19…
#>  4     0 Pinus veitch… 1.34e6  1450       181 "Familia: Pinaceae Ge… 2019-07-19…
#>  5     0 Pinus pumila  7.35e4  1395       189 "Pinaceae Genus: <spa… 2019-07-14…
#>  6     0 Pinus subg. … 3.01e5   358        27 "Pinaceae Genus: <spa… 2019-11-24…
#>  7     0 Pinus clausa  4.50e4  1552       208 "Pinaceae Genus: <spa… 2019-08-15…
#>  8     0 Pinus pseudo… 1.48e6  2114       310 "Genus: <span class=\… 2020-05-21…
#>  9     0 Pinus pinast… 1.32e6  2764       379 "Pinaceae Genus: <spa… 2019-12-20…
#> 10     0 Pinus nigra … 3.27e5  1799       138 "Genus: <span class=\… 2020-03-02…
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wikicommons.R
\name{wt_wikicommons}
\alias{wt_wikicommons}
\alias{wt_wikicommons_parse}
\alias{wt_wikicommons_search}
\title{WikiCommons}
\usage{
wt_wikicommons(name, utf8 = TRUE, ...)

wt_wikicommons_parse(
  page,
  types = c("langlinks", "iwlinks", "externallinks", "common_names", "classification"),
  tidy = FALSE
)

wt_wikicommons_search(query, limit = 10, offset = 0, utf8 = TRUE, ...)
}
\arguments{
\item{name}{(character) Wiki name - as a page title, must be length 1}

\item{utf8}{(logical) If \code{TRUE}, encodes most (but not all) non-ASCII
characters as UTF-8 instead of replacing them with hexadecimal escape
sequences. Default: \code{TRUE}}

\item{...}{curl options, passed on to \code{\link[httr:GET]{httr::GET()}}}

\item{page}{(\code{\link[httr:response]{httr::response()}}) Result of \code{\link[=wt_wiki_page]{wt_wiki_page()}}}

\item{types}{(character) List of properties to parse}

\item{tidy}{(logical). tidy output to data.frame's if possible.
Default: \code{FALSE}}

\item{query}{(character) query terms}

\item{limit}{(integer) number of results to return. Default: 10}

\item{offset}{(integer) record to start at. Default: 0}
}
\value{
\code{wt_wikicommons} returns a list, with slots:
\itemize{
\item langlinks - language page links
\item externallinks - external links
\item common_names - a data.frame with \code{name} and \code{language} columns
\item classification - a data.frame with \code{rank} and \code{name} columns
}

\code{wt_wikicommons_parse} returns a list

\code{wt_wikicommons_search} returns a list with slots for \code{continue} and
\code{query}, where \code{query} holds the results, with \code{query$search} slot with
the search results
}
\description{
WikiCommons
}
\examples{
\dontrun{
# high level
wt_wikicommons(name = "Malus domestica")
wt_wikicommons(name = "Pinus contorta")
wt_wikicommons(name = "Ursus americanus")
wt_wikicommons(name = "Balaenoptera musculus")

wt_wikicommons(name = "Category:Poeae")
wt_wikicommons(name = "Category:Pinaceae")

# low level
pg <- wt_wiki_page("https://commons.wikimedia.org/wiki/Malus_domestica")
wt_wikicommons_parse(pg)

# search wikicommons
# FIXME: utf=FALSE for now until curl::curl_escape fix 
# https://github.com/jeroen/curl/issues/228
wt_wikicommons_search(query = "Pinus", utf8 = FALSE)

## use search results to dig into pages
res <- wt_wikicommons_search(query = "Pinus", utf8 = FALSE)
lapply(res$query$search$title[1:3], wt_wikicommons)
}
}
\references{
\url{https://www.mediawiki.org/wiki/API:Search} for help on search
}
\concept{Wikicommons functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wikitaxa-package.R
\docType{data}
\name{wikipedias}
\alias{wikipedias}
\title{List of Wikipedias}
\description{
data.frame of 295 rows, with 3 columns:
\itemize{
\item language - language
\item language_local - language in local name
\item wiki - langugae code for the wiki
}
}
\details{
From \url{https://meta.wikimedia.org/wiki/List_of_Wikipedias}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wikitaxa-package.R
\docType{package}
\name{wikitaxa-package}
\alias{wikitaxa-package}
\alias{wikitaxa}
\title{wikitaxa}
\description{
Taxonomic Information from Wikipedia
}
\seealso{
Useful links:
\itemize{
  \item \url{https://docs.ropensci.org/wikitaxa/}
  \item \url{https://github.com/ropensci/wikitaxa}
  \item Report bugs at \url{https://github.com/ropensci/wikitaxa/issues}
}

}
\author{
Scott Chamberlain \email{myrmecocystus@gmail.com}

Ethan Welty
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wikipages.R
\name{wt_wiki_page}
\alias{wt_wiki_page}
\title{Get MediaWiki Page from API}
\usage{
wt_wiki_page(url, ...)
}
\arguments{
\item{url}{(character) MediaWiki page url.}

\item{...}{Arguments passed to \code{\link[=wt_wiki_url_build]{wt_wiki_url_build()}} if \code{url}
is a static page url.}
}
\value{
an \code{HttpResponse} response object from \pkg{crul}
}
\description{
Supports both static page urls and their equivalent API calls.
}
\details{
If the URL given is for a human readable html page,
we convert it to equivalent API call - if URL is already an API call,
we just use that.
}
\examples{
\dontrun{
wt_wiki_page("https://en.wikipedia.org/wiki/Malus_domestica")
}
}
\seealso{
Other MediaWiki functions: \code{\link{wt_wiki_page_parse}},
  \code{\link{wt_wiki_url_build}},
  \code{\link{wt_wiki_url_parse}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wiki.R
\name{wt_data}
\alias{wt_data}
\alias{wt_data_id}
\title{Wikidata taxonomy data}
\usage{
wt_data(x, property = NULL, ...)

wt_data_id(x, language = "en", limit = 10, ...)
}
\arguments{
\item{x}{(character) a taxonomic name}

\item{property}{(character) a property id, e.g., P486}

\item{...}{curl options passed on to \code{httr::GET()}}

\item{language}{(character) two letter language code}

\item{limit}{(integer) records to return. Default: 10}
}
\value{
\code{wt_data} searches Wikidata, and returns a list with elements:
\itemize{
\item labels - data.frame with columns: language, value
\item descriptions - data.frame with columns: language, value
\item aliases - data.frame with columns: language, value
\item sitelinks - data.frame with columns: site, title
\item claims - data.frame with columns: claims, property_value,
property_description, value (comma separated values in string)
}

\code{wt_data_id} gets the Wikidata ID for the searched term, and
returns the ID as character
}
\description{
Wikidata taxonomy data
}
\details{
Note that \code{wt_data} can take a while to run since when fetching
claims it has to do so one at a time for each claim

You can search things other than taxonomic names with \code{wt_data} if you
like
}
\examples{
\dontrun{
# search by taxon name
wt_data("Mimulus alsinoides")

# choose which properties to return
wt_data(x="Mimulus foliatus", property = c("P846", "P815"))

# get a taxonomic identifier
wt_data_id("Mimulus foliatus")

# the id can be passed directly to wt_data()
wt_data(wt_data_id("Mimulus foliatus"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wikipages.R
\name{wt_wiki_page_parse}
\alias{wt_wiki_page_parse}
\title{Parse MediaWiki Page}
\usage{
wt_wiki_page_parse(
  page,
  types = c("langlinks", "iwlinks", "externallinks"),
  tidy = FALSE
)
}
\arguments{
\item{page}{(\link[crul:HttpResponse]{crul::HttpResponse}) Result of \code{\link[=wt_wiki_page]{wt_wiki_page()}}}

\item{types}{(character) List of properties to parse.}

\item{tidy}{(logical). tidy output to data.frames when possible.
Default: \code{FALSE}}
}
\value{
a list
}
\description{
Parses common properties from the result of a MediaWiki API page call.
}
\details{
Available properties currently not parsed:
title, displaytitle, pageid, revid, redirects, text, categories,
links, templates, images, sections, properties, ...
}
\examples{
\dontrun{
pg <- wt_wiki_page("https://en.wikipedia.org/wiki/Malus_domestica")
wt_wiki_page_parse(pg)
}
}
\seealso{
Other MediaWiki functions: 
\code{\link{wt_wiki_page}()},
\code{\link{wt_wiki_url_build}()},
\code{\link{wt_wiki_url_parse}()}
}
\concept{MediaWiki functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wikipages.R
\name{wt_wiki_url_build}
\alias{wt_wiki_url_build}
\title{Build MediaWiki Page URL}
\usage{
wt_wiki_url_build(wiki, type = NULL, page = NULL, api = FALSE,
  action = "parse", redirects = TRUE, format = "json", utf8 = TRUE,
  prop = c("text", "langlinks", "categories", "links", "templates", "images",
  "externallinks", "sections", "revid", "displaytitle", "iwlinks",
  "properties"))
}
\arguments{
\item{wiki}{(character | list) Either the wiki name or a list with
\code{$wiki}, \code{$type}, and \code{$page} (the output of \code{\link[=wt_wiki_url_parse]{wt_wiki_url_parse()}}).}

\item{type}{(character) Wiki type.}

\item{page}{(character) Wiki page title.}

\item{api}{(boolean) Whether to return an API call or a static page url
(default). If \code{FALSE}, all following (API-only) arguments are ignored.}

\item{action}{(character) See \url{https://en.wikipedia.org/w/api.php}
for supported actions. This function currently only supports "parse".}

\item{redirects}{(boolean) If the requested page is set to a redirect,
resolve it.}

\item{format}{(character) See \url{https://en.wikipedia.org/w/api.php}
for supported output formats.}

\item{utf8}{(boolean) If \code{TRUE}, encodes most (but not all) non-ASCII
characters as UTF-8 instead of replacing them with hexadecimal escape
sequences.}

\item{prop}{(character) Properties to retrieve, either as a character vector
or pipe-delimited string. See
\url{https://en.wikipedia.org/w/api.php?action=help&modules=parse} for
supported properties.}
}
\value{
a URL (character)
}
\description{
Builds a MediaWiki page url from its component parts (wiki name, wiki type,
and page title). Supports both static page urls and their equivalent API
calls.
}
\examples{
wt_wiki_url_build(wiki = "en", type = "wikipedia", page = "Malus domestica")
wt_wiki_url_build(
  wt_wiki_url_parse("https://en.wikipedia.org/wiki/Malus_domestica"))
wt_wiki_url_build("en", "wikipedia", "Malus domestica", api = TRUE)
}
\seealso{
Other MediaWiki functions: \code{\link{wt_wiki_page_parse}},
  \code{\link{wt_wiki_page}},
  \code{\link{wt_wiki_url_parse}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wikipages.R
\name{wt_wiki_url_parse}
\alias{wt_wiki_url_parse}
\title{Parse MediaWiki Page URL}
\usage{
wt_wiki_url_parse(url)
}
\arguments{
\item{url}{(character) MediaWiki page url.}
}
\value{
a list with elements:
\itemize{
\item wiki - wiki language
\item type - wikipedia type
\item page - page name
}
}
\description{
Parse a MediaWiki page url into its component parts (wiki name, wiki type,
and page title). Supports both static page urls and their equivalent API
calls.
}
\examples{
wt_wiki_url_parse(url="https://en.wikipedia.org/wiki/Malus_domestica")
wt_wiki_url_parse("https://en.wikipedia.org/w/api.php?page=Malus_domestica")
}
\seealso{
Other MediaWiki functions: 
\code{\link{wt_wiki_page}()},
\code{\link{wt_wiki_page_parse}()},
\code{\link{wt_wiki_url_build}()}
}
\concept{MediaWiki functions}
